import {
  select,
  csv,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  scaleOrdinal,
  schemeCategory10
} from 'd3';
import { loadAndProcessData } from './loadAndProcessData';
import { colorLegend } from './colorLegend';
import { scatterplot } from './scatterplot';
import { dropdownMenu } from './dropdownMenu';
import { parallelCoordinates } from './parallelCoordinates';
import { tabulate, updateTabulate } from './tabulate';

const svg = select('svg');

const parallel_coordinates = select('#parallel_coordinates');

const width = +svg.attr('width');
const height = +svg.attr('height');

const colorLegendG = svg.append('g');

const color = scaleOrdinal(schemeCategory10);

let selectedCategory = null;
let selectedItems = new Set();
let selectedItem = null;
let mouseHoverItem = null;
let menuItems;
let columns;
let xColumn;
let yColumn;

const colorValue = data => data.Category;

function isBrushed(brush_coords, cx, cy) {

    var x0 = brush_coords[0][0],
      x1 = brush_coords[1][0],
      y0 = brush_coords[0][1],
      y1 = brush_coords[1][1];

    return x0 <= cx && cx <= x1 && y0 <= cy && cy <= y1;
  }

function highlightBrushedCircles() {
  	let circles = svg.selectAll('circle').data(menuItems);
    if (d3.event.selection != null) {
    
      // revert circles to initial style
      circles.attr("class", "non_brushed");

      var brush_coords = d3.brushSelection(this);

      // style brushed circles
      circles.filter(function () {

        var cx = d3.select(this).attr("cx"),
          cy = d3.select(this).attr("cy");
				// console.log(isBrushed(brush_coords, cx, cy));
        return isBrushed(brush_coords, cx, cy);
      })
        .attr("class", "brushed");
      
      let brushedItems = d3.selectAll(".brushed").data().length > 0 ? d3.selectAll(".brushed").data() : menuItems;
    
      render();
      updateTable(brushedItems);
    }
  }

const onTableRowMouseOver = function (d) {
  d3.selectAll(".highlighted")
    .classed("highlighted", false);
  d3.select(this)
    .attr('class', 'highlighted');
  mouseHoverItem = d;
  // console.log(mouseHoverItem);
  render();
};

const onTableRowMouseLeave = d => {
  mouseHoverItem = null;
  render();
};

const onClick = d => {
  mouseHoverItem = null;
  selectedItem = null;
  selectedCategory = d;
  updateTable(filterMenuByCategory(selectedCategory));
  render();
};

const updateTable = data => {
  if(data === null || data.length === 0) 
    data = menuItems;
  d3.select('#grid table').call(updateTabulate, {
      data, 
      columns,
      onTableRowMouseOver
    });
};

const resetAllChart = d => {
  d3.selectAll(".highlighted")
    	.classed("highlighted", false);
  mouseHoverItem = null;
  selectedItem = null;
  selectedCategory = null;
  updateTable();
  render();
};

const resetBtn = d3.select('#menus').append('button')
											.attr('width', '200px')
											.attr('height', '200px')
											.text('reset')
											.on('click', resetAllChart);

loadAndProcessData().then(data => {
  menuItems = data;
  columns = data.columns;
  xColumn = data.columns[3];
  yColumn = data.columns[19];
  d3.select('#grid').call(tabulate, {
    data: menuItems,
    columns,
    onTableRowMouseOver,
    onTableRowMouseLeave,
    resetAllChart
  });
  render();
});

const onXColumnClicked = column => {
  xColumn = column;
  render();
};

const onYColumnClicked = column => {
  yColumn = column;
  render();
};

const onItemClicked = item => {
  // if(selectedItems.has(item))
  //   selectedItems.delete(item);
  // else
  //   selectedItems.add(item);
  // select('#total').text(`Total Calories: ${calculateCalories()}`);
  // console.log(selectedItems);
  if (item === selectedItem)
    selectedItem = null;
  else
    selectedItem = item;
  render();
};

const calculateCalories = () => {
  var totalCalories = 0;
  selectedItems.forEach(item => {
    totalCalories += item['Calories'];
  });
  return totalCalories;
};

// tooltip mouseover event handler
var tipMouseover = function (d) {
  var html = d.Item + "<br/>" +
    "<span style='color:" + color(colorValue(d)) + ";'>" + d.Category + "</span><br/>" +
    "<b>" + d[xColumn] + `</b> ${xColumn}, <b/>` + d[yColumn] + `</b> ${yColumn}`;
  select('.tooltip').html(html)
    .style("left", (d3.event.pageX + 15) + "px")
    .style("top", (d3.event.pageY - 28) + "px")
    .transition()
    .duration(200) // ms
    .style("opacity", .9) // started as 0!
};
// tooltip mouseout event handler
var tipMouseout = function (d) {
  select('.tooltip').transition()
    .duration(300) // ms
    .style("opacity", 0);
};

var filterMenuByCategory = category => {
  return menuItems.filter(item => item['Category'] === category);
}

const render = () => {

  select('#x-menu')
    .call(dropdownMenu, {
      options: menuItems.columns.filter(column =>
        column !== 'Item' &&
        column !== 'Category'
      ),
      onOptionClicked: onXColumnClicked,
      selectedOption: xColumn
    });

  select('#y-menu')
    .call(dropdownMenu, {
      options: menuItems.columns,
      onOptionClicked: onYColumnClicked,
      selectedOption: yColumn
    });

  svg.call(scatterplot, {
    xAxisLabel: xColumn,
    yAxisLabel: yColumn,
    xValue: d => d[xColumn],
    yValue: d => d[yColumn],
    circleRadius: 10,
    margin: { top: 10, right: 185, bottom: 88, left: 150 },
    width,
    height,
    data: menuItems,
    color,
    colorValue,
    selectedCategory,
    tipMouseover,
    tipMouseout,
    selectedItem,
    onItemClicked,
    highlightBrushedCircles
  });


  svg.call(colorLegend, {
    color,
    width,
    selectedCategory,
    onClick
  });

  d3.select('.parcoords').call(parallelCoordinates, {
    margin: { top: 80, right: 185, bottom: 88, left: 150 },
    width,
    height,
    data: menuItems,
    color,
    selectedCategory,
    selectedItem,
    columns,
    mouseHoverItem,
    onTableRowMouseOver,
    updateTable,
    filterMenuByCategory
  })

  // d3.select('#grid').call(tabulate, {
  //   data: menuItems,
  //   columns,
  //   onTableRowMouseOver
  // });

  select('body').append("div")
    .attr("class", "tooltip")
    .style("opacity", 0);


  // d3.selectAll('tr').on('click', d => console.log(d));

  /*var slider2 = d3.sliderHorizontal()
    .min(0)
    .max(10)
    .step(1)
    .width(400)
    .on('onchange', val => {
      console.log(val);
      //d3.select("p#value2").text(val);
    });

  var g = d3.select("#slider").append("svg")
    .attr("width", 600)
    .attr("height", 100)
    .append("g")
    .attr("transform", "translate(30,30)");
  
  g.call(slider2);*/

};
